@@ -7,6 +7,7 @@ import base64
 import httplib
 import re
 import copy
+import UserDict
 
 # Use core json for 2.6+, simplejson for <=2.5
 try:
@@ -37,6 +38,73 @@ __organization__ = "InterMine"
 __license__ = "LGPL"
 __contact__ = "dev@intermine.org"
 
+class Registry(object, UserDict.DictMixin):
+    """
+    A Class representing an InterMine registry.
+    ===========================================
+
+    Registries are web-services that mines can automatically register themselves
+    with, and thus enable service discovery by clients.
+
+    SYNOPSIS
+    --------
+
+    example::
+
+        from intermine.webservice import Registry
+
+        # Connect to the default registry service
+        # at www.intermine.org/registry
+        registry = Registry()
+
+        # Find all the available mines:
+        for name, mine in registry.items():
+            print name, mine.version
+
+        # Dict-like interface for accessing mines.
+        flymine = registry["flymine"]
+
+        # The mine object is a Service
+        for gene in flymine.select("Gene.*").results():
+            process(gene)
+
+    This class is meant to aid with interoperation between
+    mines by allowing them to discover one-another, and 
+    allow users to always have correct connection information.
+    """
+
+    MINES_PATH = "/mines.json"
+
+    def __init__(self, registry_url="http://www.intermine.org/registry"):
+        self.registry_url = registry_url
+        opener = InterMineURLOpener()
+        data = opener.open(registry_url + Registry.MINES_PATH).read()
+        mine_data = json.loads(data)
+        self.__mine_dict = dict(( (mine["name"], mine) for mine in mine_data["mines"]))
+        self.__synonyms = dict(( (name.lower(), name) for name in self.__mine_dict.keys() ))
+        self.__mine_cache = {}
+
+    def __contains__(self, name):
+        return name.lower() in self.__synonyms
+
+    def __getitem__(self, name):
+        lc = name.lower()
+        if lc in self.__synonyms:
+            if lc not in self.__mine_cache:
+                self.__mine_cache[lc] = Service(self.__mine_dict[self.__synonyms[lc]]["webServiceRoot"])
+            return self.__mine_cache[lc]
+        else:
+            raise KeyError("Unknown mine: " + name)
+
+    def __setitem__(self, name, item):
+        raise NotImplementedError("You cannot add items to a registry")
+
+    def __delitem__(self, name): 
+        raise NotImplementedError("You cannot remove items from a registry")
+
+    def keys(self):
+        return self.__mine_dict.keys()
+
 class Service(object):
     """
     A class representing connections to different InterMine WebServices
@@ -258,7 +326,7 @@ class Service(object):
             self._release = urllib.urlopen(self.root + self.RELEASE_PATH).read()
         return self._release
 
-    def new_query(self, root=None):
+    def new_query(self, root=None, xml=None):
         """
         Construct a new Query object for the given webservice
         =====================================================
@@ -270,7 +338,10 @@ class Service(object):
 
         @return: L{intermine.query.Query}
         """
-        return Query(self.model, self, root=root)
+        if xml is not None:
+            return Query.from_xml(xml, self.model, root=root)
+        else:
+            return Query(self.model, self, root=root)
 
     def select(self, *columns):
         """
@@ -280,6 +351,11 @@ class Service(object):
         As new_query, except that instead of a root class, a list of 
         output column expressions are passed instead.
         """
+        if len(columns) == 1 and not columns[0].endswith("*"):
+            view = columns[0]
+            path = self.model.make_path(view)
+            if not path.is_attribute():
+                return Query(self.model, self).select(view + ".*")
         return Query(self.model, self).select(*columns)
 
     def get_template(self, name):
@@ -383,7 +459,7 @@ class Service(object):
         @type path: string
         @param params: The query parameters for this request as a dictionary
         @type params: dict
-        @param rowformat: One of "rr", "dict", "list", "tsv", "csv", "jsonrows", "jsonobjects"
+        @param rowformat: One of "rr", "object", "count", "dict", "list", "tsv", "csv", "jsonrows", "jsonobjects"
         @type rowformat: string
         @param view: The output columns
         @type view: list
@@ -599,6 +675,18 @@ class ResultIterator(object):
         self.rowformat = rowformat
         self._it = None
 
+    def __len__(self):
+        """
+        Return the number of items in this iterator
+        ===========================================
+
+        Note that this requires iterating over the full result set.
+        """
+        c = 0
+        for x in self:
+            c += 1
+        return c
+
     def __iter__(self):
         """
         Return an iterator over the results
