@@ -139,6 +139,86 @@ class TestQuery(WebserviceTest): # pragma: no cover
     expected_ternary = '[<TernaryConstraint: Employee LOOKUP Susan>, <TernaryConstraint: Employee.department.manager LOOKUP John IN Wernham-Hogg>]'
     expected_subclass = "[<SubClassConstraint: Department.employees ISA Manager>]"
 
+    XML_1 = """
+        <query model="testmodel" view="Employee.name Employee.age">
+        </query>
+        """
+    EXPECTED_VIEWS_1 = ["Employee.name", "Employee.age"]
+
+    XML_2 = """
+        <query model="testmodel" 
+            view="Department.employees.name Department.employees.age Department.employees.seniority"
+            >
+            <constraint path="Department.employees" type="Manager"/>
+        </query>
+        """
+
+    EXPECTED_VIEWS_2 = ["Department.employees.name",
+        "Department.employees.age", "Department.employees.seniority"]
+
+    XML_3 = """
+        <query model="testmodel" 
+            view="Department.employees.name Department.employees.age Department.employees.seniority"
+            >
+            <constraint path="Department.employees" type="Manager"/>
+            <constraint path="Department.name" op="=" value="foo"/>
+        </query>
+        """
+
+    XML_4 = """
+        <query model="testmodel" 
+            view="Department.employees.name Department.employees.age Department.employees.seniority"
+            sortOrder="Department.employees.age asc"
+            >
+            <constraint path="Department.employees" type="Manager"/>
+            <constraint path="Department.name" op="=" value="foo"/>
+        </query>
+        """
+
+    XML_5 = """
+        <query model="testmodel" 
+            view="Department.employees.name Department.employees.age Department.employees.seniority"
+            sortOrder="Department.employees.age asc"
+            >
+            <pathDescription description="Should be a manager" pathString="Department.employees"/>
+            <constraint path="Department.employees" type="Manager"/>
+            <constraint path="Department.name" op="=" value="foo"/>
+        </query>
+        """
+
+    XML_6 = """
+        <query model="testmodel" 
+            view="Department.employees.name Department.employees.age Department.employees.salary"
+            sortOrder="Department.employees.salary asc"
+            >
+            <pathDescription description="The Company" pathString="Department.employees.company"/>
+            <pathDescription description="Should be a CEO" pathString="Department.employees"/>
+            <constraint path="Department.employees" type="CEO"/>
+            <constraint path="Department.employees.company" op="LOOKUP" value="foo"/>
+        </query>
+        """
+
+    EXPECTED_VIEWS_6 = ["Department.employees.name",
+        "Department.employees.age", "Department.employees.salary"]
+
+    XML_7 = """
+        <query model="testmodel" 
+            view="Department.employees.seniority Department.employees.age Department.employees.salary"
+            sortOrder="Department.employees.salary asc"
+            >
+            <pathDescription description="The Company" pathString="Department.employees.company"/>
+            <pathDescription description="Should be a CEO" pathString="Department.employees"/>
+            <constraint path="Department.employees" type="CEO"/>
+            <constraint path="Department.employees.company" op="LOOKUP" value="foo"/>
+        </query>
+        """
+
+    EXPECTED_VIEWS_7 = ["Department.employees.seniority",
+        "Department.employees.age", "Department.employees.salary"]
+
+    CONSTRAINTS_COUNT_3 = 2
+
+
     def setUp(self):
         if self.service is None:
             self.__class__.service = Service(self.get_test_root())
@@ -150,6 +230,40 @@ class TestQuery(WebserviceTest): # pragma: no cover
         list_dict = {"service": None, "manager": DummyManager(), "name": "my-list", "title": None, "type": "Employee", "size": 10}
         self.l = List(**list_dict)
 
+    def testFromXML(self):
+        q1 = self.service.new_query(xml = TestQuery.XML_1)
+        self.assertEqual(q1.views, TestQuery.EXPECTED_VIEWS_1)
+
+        q2 = self.service.new_query(xml = TestQuery.XML_2)
+        self.assertEqual(q2.views, TestQuery.EXPECTED_VIEWS_2)
+
+        q3 = self.service.new_query(xml = TestQuery.XML_3)
+        self.assertEqual(q3.views, TestQuery.EXPECTED_VIEWS_2)
+        self.assertEqual(len(q3.constraints), TestQuery.CONSTRAINTS_COUNT_3)
+
+        q4 = self.service.new_query(xml = TestQuery.XML_4)
+        self.assertEqual(q4.views, TestQuery.EXPECTED_VIEWS_2)
+        self.assertEqual(len(q4.constraints), TestQuery.CONSTRAINTS_COUNT_3)
+        self.assertEqual(str(q4.get_sort_order()), "Department.employees.age asc")
+
+        q5 = self.service.new_query(xml = TestQuery.XML_5)
+        self.assertEqual(q5.views, TestQuery.EXPECTED_VIEWS_2)
+        self.assertEqual(len(q5.constraints), TestQuery.CONSTRAINTS_COUNT_3)
+        self.assertEqual(str(q5.get_sort_order()), "Department.employees.age asc")
+        self.assertEqual(len(q5.path_descriptions), 1)
+
+        q6 = self.service.new_query(xml = TestQuery.XML_6)
+        self.assertEqual(q6.views, TestQuery.EXPECTED_VIEWS_6)
+        self.assertEqual(len(q6.constraints), TestQuery.CONSTRAINTS_COUNT_3)
+        self.assertEqual(str(q6.get_sort_order()), "Department.employees.salary asc")
+        self.assertEqual(len(q6.path_descriptions), 2)
+
+        q7 = self.service.new_query(xml = TestQuery.XML_7)
+        self.assertEqual(q7.views, TestQuery.EXPECTED_VIEWS_7)
+        self.assertEqual(len(q7.constraints), TestQuery.CONSTRAINTS_COUNT_3)
+        self.assertEqual(str(q7.get_sort_order()), "Department.employees.salary asc")
+        self.assertEqual(len(q7.path_descriptions), 2)
+
     def testAddViews(self):
         """Queries should be able to add legal views, and complain about illegal ones"""
         self.q.add_view("Employee.age")
@@ -334,14 +448,16 @@ class TestQuery(WebserviceTest): # pragma: no cover
         except ConstraintError, ex:
             self.assertEqual(ex.message, "'Manager' is not a subclass of 'Department.company.CEO'")
 
-    def testLogic(self):
+    def testStringLogic(self):
         """Queries should be able to parse good logic strings"""
+
         a = self.q.add_constraint("Employee.name", "IS NOT NULL")
         b = self.q.add_constraint("Employee.age", ">", 10)
         c = self.q.add_constraint("Employee.department", "LOOKUP", "Sales", "Wernham-Hogg")
         d = self.q.add_constraint("Employee.department.employees.name", "ONE OF", 
             ["John", "Paul", "Mary"])
         self.q.add_constraint("Employee.department.employees", "Manager")
+        
         self.assertEqual(str(self.q.get_logic()), "A and B and C and D")
         self.q.set_logic("(B or C) and (A or D)")
         self.assertEqual(str(self.q.get_logic()), "(B or C) and (A or D)")
@@ -349,6 +465,40 @@ class TestQuery(WebserviceTest): # pragma: no cover
         self.assertEqual(str(self.q.get_logic()), "B and (C or A) and D")
         self.q.set_logic("(A and B) or (A and C and D)")
         self.assertEqual(str(self.q.get_logic()), "(A and B) or (A and C and D)")
+
+    def testIrrelevantCodeStripping(self):
+        """Should be able to recover from queries that have irrelevant codes in their logic"""
+
+        a = self.q.add_constraint("Employee.name", "IS NOT NULL")
+        b = self.q.add_constraint("Employee.age", ">", 10)
+        c = self.q.add_constraint("Employee.department", "LOOKUP", "Sales", "Wernham-Hogg")
+        d = self.q.add_constraint("Employee.department.employees.name", "ONE OF", 
+            ["John", "Paul", "Mary"])
+        self.q.add_constraint("Employee.department.employees", "Manager")
+
+        self.q._set_questionable_logic("A and B or C or D and E")
+        self.assertEqual(str(self.q.get_logic()), "A and (B or C or D)")
+        self.q._set_questionable_logic("E and A and B or C or D")
+        self.assertEqual(str(self.q.get_logic()), "A and (B or C or D)")
+        self.q._set_questionable_logic("A and B and E or C or D")
+        self.assertEqual(str(self.q.get_logic()), "A and B and (C or D)")
+        self.q._set_questionable_logic("A and B or X and J or Z and E or C or D")
+        self.assertEqual(str(self.q.get_logic()), "A and B and (C or D)")
+        self.q._set_questionable_logic("A and B or X and (J or Z and E or C) or D")
+        self.assertEqual(str(self.q.get_logic()), "(A and B and C) or D")
+        self.q._set_questionable_logic("A or (B or X and J or Z and E or C) or D")
+        self.assertEqual(str(self.q.get_logic()), "A or (B and C) or D")
+
+    def testObjectLogic(self):
+        """Queries should be able to set logic from object methods"""
+
+        a = self.q.add_constraint("Employee.name", "IS NOT NULL")
+        b = self.q.add_constraint("Employee.age", ">", 10)
+        c = self.q.add_constraint("Employee.department", "LOOKUP", "Sales", "Wernham-Hogg")
+        d = self.q.add_constraint("Employee.department.employees.name", "ONE OF", 
+            ["John", "Paul", "Mary"])
+        self.q.add_constraint("Employee.department.employees", "Manager")
+
         self.q.set_logic(a + b + c + d)
         self.assertEqual(str(self.q.get_logic()), "A and B and C and D")
         self.q.set_logic(a & b & c & d)
@@ -819,9 +969,6 @@ class TestResultObjects(WebserviceTest): # pragma: no cover
                 assertEqual(len(departments[-1].employees), 5)
 
                 for idx in [0, -1]:
-                    #assertEqual(departments[idx].manager, None) # Unrequested refs are none, even if they would otherwise have had a value
-                    #assertEqual(departments[idx].company.name, None) # Unrequested attrs are none
-                    #assertEqual(departments[idx].company.contractors, []) # Unrequested collections are empty
                     self.assertRaises(ModelError, lambda: departments[idx].foo) # Model errors are thrown for illegal field access
                     self.assertRaises(ModelError, lambda: departments[idx].company.foo)
 
